unit LEDsMain;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Buttons, StdCtrls, JvHidControllerClass, JvComponentBase;

type
  // a buffer to read and write bytes to device
  // [0] = ReportID (always 0)
  TDeviceBuffer = array [0..63] of Byte;

  TLEDs = class(TForm)
    SpeedButton1: TSpeedButton;
    Edit1: TEdit;
    ClearAll: TButton;
    SetAll: TButton;
    Read: TButton;
    JvHidDeviceController1: TJvHidDeviceController;
    procedure FormCreate(Sender: TObject);
    procedure ClearAllClick(Sender: TObject);
    procedure SetAllClick(Sender: TObject);
    procedure ReadClick(Sender: TObject);
    procedure SpeedButton1Click(Sender: TObject);
    procedure Edit1Change(Sender: TObject);
  public
    AllButtons: Boolean;
    AllEdits: Boolean;
    LEDs:  array [0..7, 0..7] of TSpeedButton;
    Edits: array [0..7] of TEdit;
    WriteBytes: TDeviceBuffer;
    ReadBytes: TDeviceBuffer;
    BytesRead: Cardinal;
    function EnumerateAndWrite(HidDev: TJvHidDevice; const Idx: Integer): Boolean;
    function EnumerateAndRead(HidDev: TJvHidDevice; const Idx: Integer): Boolean;
  end;

var
  LEDs: TLEDs;

implementation

{$R *.DFM}

// on program start

procedure TLEDs.FormCreate(Sender: TObject);
var
  I, J: Integer;
begin
  // template controls
  LEDs[0, 0] := SpeedButton1;
  Edits[0] := Edit1;

  // create first row of LEDs and Edits from templates
  for I := 1 to 7 do
  begin
    LEDs[I, 0] := TSpeedButton.Create(Self);
    LEDs[I, 0].Parent     := SpeedButton1.Parent;
    LEDs[I, 0].Left       := SpeedButton1.Left + I * SpeedButton1.Width;
    LEDs[I, 0].Top        := SpeedButton1.Top;
    LEDs[I, 0].Glyph      := SpeedButton1.Glyph;
    LEDs[I, 0].AllowAllUp := True;
    LEDs[I, 0].GroupIndex := I+1;
    LEDs[I, 0].Tag        := I;
    LEDs[I, 0].OnClick    := SpeedButton1.OnClick;
  
    Edits[I] := TEdit.Create(Self);
    Edits[I].Parent    := Edit1.Parent;
    Edits[I].Left      := Edit1.Left;
    Edits[I].Width     := Edit1.Width;
    Edits[I].MaxLength := Edit1.MaxLength;
    Edits[I].Top       := Edits[I-1].Top + SpeedButton1.Height;
    Edits[I].Text      := '00';
    Edits[I].Tag       := I;
    Edits[I].TabOrder  := I;
    Edits[I].OnChange  := Edit1.OnChange;
  end;

  // create the other rows of LEDs
  for J := 1 to 7 do
    for I := 0 to 7 do
    begin
      LEDs[I, J] := TSpeedButton.Create(Self);
      LEDs[I, J].Parent     := SpeedButton1.Parent;
      LEDs[I, J].Left       := LEDs[I, J-1].Left;
      LEDs[I, J].Top        := LEDs[I, J-1].Top  + SpeedButton1.Height;
      LEDs[I, J].Glyph      := SpeedButton1.Glyph;
      LEDs[I, J].AllowAllUp := True;
      LEDs[I, J].GroupIndex := J*8+I+10;
      LEDs[I, J].Tag        := J*8+I;
      LEDs[I, J].OnClick    := SpeedButton1.OnClick;
    end;
  // initialize Report buffers
  FillChar(WriteBytes, SizeOf(WriteBytes), 0);
  FillChar(ReadBytes,  SizeOf(ReadBytes),  0);
end;

// Clear All button clicked

procedure TLEDs.ClearAllClick(Sender: TObject);
var
  I, J: Integer;
begin
  AllButtons := True;
  // set all LEDs to off
  for I := 0 to 7 do
  begin
    for J := 0 to 7 do
      if LEDs[I, J].Down then
      begin
        LEDs[I, J].Down := False;
        LEDs[I, J].OnClick(LEDs[I, J]);
      end;
    // set data
    WriteBytes[I+1] := 0;
  end;
  // write data to device
  JvHidDeviceController1.OnEnumerate := EnumerateAndWrite;
  JvHidDeviceController1.Enumerate;
  AllButtons := False;
end;

// Set All button clicked

procedure TLEDs.SetAllClick(Sender: TObject);
var
  I, J: Integer;
begin
  AllButtons := True;
  // set all LEDs to on
  for I := 0 to 7 do
  begin
    for J := 0 to 7 do
      if not LEDs[I, J].Down then
      begin
        LEDs[I, J].Down := True;
        LEDs[I, J].OnClick(LEDs[I, J]);
      end;
    // set data
    WriteBytes[I+1] := $FF;
  end;
  // write data to device
  JvHidDeviceController1.OnEnumerate := EnumerateAndWrite;
  JvHidDeviceController1.Enumerate;
  AllButtons := False;
end;

// Read button clicked

procedure TLEDs.ReadClick(Sender: TObject);
var
  I, J: Integer;
begin
  // try to read data from a device
  BytesRead := 0;
  JvHidDeviceController1.OnEnumerate := EnumerateAndRead;
  JvHidDeviceController1.Enumerate;
  // if bytes read
  if BytesRead > 0 then
  begin
    AllButtons := True;
    // place data in LEDs and Edits
    for I := 0 to 7 do
    begin
      for J := 0 to 7 do
      begin
        LEDs[J, I].Down := (ReadBytes[I+1] and (1 shl J)) <> 0;
        LEDs[J, I].OnClick(LEDs[J, I]);
      end;
    end;
    AllButtons := False;
    // Sync read and write buffer
    WriteBytes := ReadBytes;
  end;
end;

// a LED button clicked

procedure TLEDs.SpeedButton1Click(Sender: TObject);
var
  Val: Cardinal;
begin
  // only handle if not an OnClick call from Edit changes
  if not AllEdits then
    with Sender as TSpeedButton do
    begin
      // get current data from the Edit
      try
        Val := StrToInt('$'+Edits[Tag div 8].Text);
      except
        Val := 0;
      end;
      // add new data from click
      if Down then
        Val := Val or (1 shl (Tag mod 8))
      else
        Val := Val and not (1 shl (Tag mod 8));
      Edits[Tag div 8].Text := Format('%.2x', [Val]);
      WriteBytes[Tag div 8 + 1] := Val;
      // only write if not in All LEDs write situation
      if not AllButtons then
      begin
        // write data to device
        JvHidDeviceController1.OnEnumerate := EnumerateAndWrite;
        JvHidDeviceController1.Enumerate;
      end;
    end;
end;

procedure TLEDs.Edit1Change(Sender: TObject);
var
  I: Integer;
  Val: Cardinal;
begin
  // only handle if not called by an ALL button click
  if not AllButtons then
    with Sender as TEdit do
    begin
      // get data from Edit
      try
        Val := StrToInt('$'+Edits[Tag].Text);
      except
        Val := 0;
      end;
      AllEdits := True;
      // put data to the LED row
      for I := 0 to 7 do
      begin
        LEDs[I, Tag].Down := (Val and (1 shl I)) <> 0;
        LEDs[I, Tag].OnClick(LEDs[Tag, I]);
      end;
      WriteBytes[Tag+1] := Val;
      // write data to device
      JvHidDeviceController1.OnEnumerate := EnumerateAndWrite;
      JvHidDeviceController1.Enumerate;
      AllEdits := False;
    end;
end;

function TLEDs.EnumerateAndWrite(HidDev: TJvHidDevice; const Idx: Integer): Boolean;
var
  BytesWritten: Cardinal;
begin
  // this is the ReportID byte which precedes any report
  WriteBytes[0] := 0;
  // we wait for the correct device VID/PID = 07C0/0140..15F and
  // Usage = system device   not keyboard
  if (HidDev.Attributes.VendorID = $07C0)   and (HidDev.Attributes.ProductID >= $0140) and
     (HidDev.Attributes.ProductID <= $015F) and (HidDev.Caps.Usage = $80) then
    HidDev.WriteFile(WriteBytes[0], HidDev.Caps.OutputReportByteLength, BytesWritten);
  // do not stop enumeration  write to all devices present
  Result := True;
end;

function TLEDs.EnumerateAndRead(HidDev: TJvHidDevice; const Idx: Integer): Boolean;
begin
  Result := True;
  // this is the ReportID byte which precedes any report
  FillChar(ReadBytes, SizeOf(ReadBytes), $00);
  // we wait for the correct device VID/PID = 07C0/0140..15F and
  // Usage = system device   not keyboard
  if (HidDev.Attributes.VendorID = $07C0)   and (HidDev.Attributes.ProductID >= $0140) and
     (HidDev.Attributes.ProductID <= $015F) and (HidDev.Caps.Usage = $80) then
  begin
    HidDev.ReadFile(ReadBytes[0], HidDev.Caps.InputReportByteLength, BytesRead);
    // stop enumeration  read only from the first matching device
    Result := False;
  end;
end;

end.
